@@ -12,6 +12,7 @@ from typing import (
     Iterator,
     List,
     TYPE_CHECKING,
+    Tuple,
 )
 
 if TYPE_CHECKING:
@@ -25,6 +26,7 @@ except:
 K = TypeVar("K")
 V = TypeVar("V")
 T = TypeVar("T")
+R = TypeVar("R")
 
 
 class SuperDict(dict, Generic[K, V], Mapping[K, V]):
@@ -38,25 +40,58 @@ class SuperDict(dict, Generic[K, V], Mapping[K, V]):
     def __iter__(self) -> Iterator[K]:
         return dict.__iter__(self)
 
-    def __add__(self, other):
+    def __add__(self, other: Union[dict, int, float, str]) -> "SuperDict":
+        """
+        Applies the adding operator to the values in the SuperDict and another object.
+        This operation might raise a TypeError based on the types of the values of the SuperDict and the other object.
+        """
         return self.sapply(op.__add__, other)
 
     # def __radd__(self, other):
     #     return self.sapply(op.__add__, other)
 
-    def __sub__(self, other):
+    def __sub__(self, other: Union[dict, int, float, str]) -> "SuperDict":
+        """
+        Applies the substract operator to the values in the SuperDict and another object.
+        This operation might raise a TypeError based on the types of the values of the SuperDict and the other object.
+        """
         return self.sapply(op.__sub__, other)
 
     # def __rsub__(self, other):
     #     return other.sapply(op.__sub__, self)
 
-    def __mul__(self, other):
+    def __mul__(self, other: Union[dict, int, float, str]) -> "SuperDict":
+        """
+        Applies the multiply operator to the values in the SuperDict and another object.
+        This operation might raise a TypeError based on the types of the values of the SuperDict and the other object.
+        """
         return self.sapply(op.__mul__, other)
 
     # def __rmul__(self, other):
     #     return self.sapply(op.__mul__, other)
 
-    def head(self):
+    def __truediv__(self, other: Union[dict, int, float, str]) -> "SuperDict":
+        """
+        Applies the true division (float division) operator to the values in the SuperDict and another object.
+        This operation might raise a TypeError based on the types of the values of the SuperDict and the other object.
+        """
+        return self.sapply(op.__truediv__, other)
+
+    def __floordiv__(self, other: Union[dict, int, float, str]) -> "SuperDict":
+        """
+        Applies the floor division (integer division) operator to the values in the SuperDict and another object.
+        This operation might raise a TypeError based on the types of the values of the SuperDict and the other object.
+        """
+        return self.sapply(op.__floordiv__, other)
+
+    def head(self) -> str:
+        """
+        Returns a string representation with the first pair of key values in the SuperDict, the last pair of key value
+        and the number of elements on the SuperDict.
+
+        :return: the head string representation of the SuperDict
+        :rtype: str
+        """
         if len(self) <= 2:
             return dict.__repr__(self)
         _keys = self.keys_l()
@@ -76,7 +111,7 @@ class SuperDict(dict, Generic[K, V], Mapping[K, V]):
             return val_list
         return val_list[pos]
 
-    def keys_l(self, pos: int = None) -> list:
+    def keys_l(self, pos: int = None) -> List[K]:
         """
         Shortcut to:
 
@@ -89,7 +124,7 @@ class SuperDict(dict, Generic[K, V], Mapping[K, V]):
         result = list(self.keys())
         return self._list_or_value(result, pos)
 
-    def values_l(self, pos: int = None) -> list:
+    def values_l(self, pos: int = None) -> List[V]:
         """
         Shortcut to:
 
@@ -102,7 +137,7 @@ class SuperDict(dict, Generic[K, V], Mapping[K, V]):
         result = list(self.values())
         return self._list_or_value(result, pos)
 
-    def values_tl(self, pos: int = None) -> "TupList":
+    def values_tl(self, pos: int = None) -> "TupList[V]":
         """
         Shortcut to:
 
@@ -117,7 +152,7 @@ class SuperDict(dict, Generic[K, V], Mapping[K, V]):
         result = tl.TupList(self.values())
         return self._list_or_value(result, pos)
 
-    def keys_tl(self, pos: int = None) -> "TupList":
+    def keys_tl(self, pos: int = None) -> "TupList[K]":
         """
         Shortcut to:
 
@@ -132,7 +167,7 @@ class SuperDict(dict, Generic[K, V], Mapping[K, V]):
         result = tl.TupList(self.keys())
         return self._list_or_value(result, pos)
 
-    def items_tl(self, pos: int = None) -> "TupList":
+    def items_tl(self, pos: int = None) -> "TupList[Tuple[K, V]]":
         """
         Shortcut to:
 
@@ -153,7 +188,7 @@ class SuperDict(dict, Generic[K, V], Mapping[K, V]):
 
         :param default_value: value of elements to take out
         :param function func: function that evaluates to true if we take out the element
-        :param **kwags: optional arguments for func
+        :param kwargs: optional arguments for func
         :return: new :py:class:`SuperDict`
         :rtype: :py:class:`SuperDict`
 
@@ -164,7 +199,7 @@ class SuperDict(dict, Generic[K, V], Mapping[K, V]):
             func = lambda x: x != default_value
         return self.vfilter(func=func, **kwargs)
 
-    def vfilter(self, func: Callable, **kwargs) -> "SuperDict":
+    def vfilter(self, func: Callable, **kwargs) -> "SuperDict[K, V]":
         """
         apply a filter over the dictionary values
         :param function func: True for values we want to filter
@@ -179,7 +214,7 @@ class SuperDict(dict, Generic[K, V], Mapping[K, V]):
             {key: value for key, value in self.items() if func(value, **kwargs)}
         )
 
-    def kfilter(self, func: Callable, **kwargs) -> "SuperDict":
+    def kfilter(self, func: Callable, **kwargs) -> "SuperDict[K, V]":
         """
         apply a filter over the dictionary keys
         :param function func: True for keys we want to filter
@@ -194,7 +229,7 @@ class SuperDict(dict, Generic[K, V], Mapping[K, V]):
             {key: value for key, value in self.items() if func(key, **kwargs)}
         )
 
-    def kvfilter(self, func: Callable, **kwargs) -> "SuperDict":
+    def kvfilter(self, func: Callable, **kwargs) -> "SuperDict[K, V]":
         """
         apply a filter over the dictionary values
         :param callable func: True for values we want to filter
@@ -220,7 +255,7 @@ class SuperDict(dict, Generic[K, V], Mapping[K, V]):
         """
         return len(self)
 
-    def filter(self, indices: Iterable, check: bool = True) -> "SuperDict":
+    def filter(self, indices: Iterable, check: bool = True) -> "SuperDict[K, V]":
         """
         takes out elements that are not in `indices`
 
@@ -321,14 +356,16 @@ class SuperDict(dict, Generic[K, V], Mapping[K, V]):
         """
         return SuperDict().dicts_to_tup([], self)
 
-    def list_reverse(self) -> "SuperDict":
+    def list_reverse(self) -> "SuperDict[V, TupList[K]]":
         """
         transforms dictionary of lists to another dictionary of lists only indexed by the values.
 
         :return: new :py:class:`SuperDict`
         """
+        from . import tuplist as tl
+
         new_keys = list(set(val for l in self.values() for val in l))
-        dict_out = SuperDict({k: [] for k in new_keys})
+        dict_out = SuperDict({k: tl.TupList() for k in new_keys})
         for k, v in self.items():
             for el in v:
                 dict_out[el].append(k)
@@ -380,7 +417,7 @@ class SuperDict(dict, Generic[K, V], Mapping[K, V]):
             {key: value[property] for key, value in self.items() if property in value}
         )
 
-    def to_lendict(self) -> "SuperDict":
+    def to_lendict(self) -> "SuperDict[K, int]":
         """
         get length of values in dictionary
 
@@ -426,7 +463,7 @@ class SuperDict(dict, Generic[K, V], Mapping[K, V]):
             return result.values_l()
         return result
 
-    def kvapply(self, func: Callable, *args, **kwargs) -> "SuperDict":
+    def kvapply(self, func: Callable[[K, V], R], *args, **kwargs) -> "SuperDict[K, R]":
         """Applies a function to the dictionary and returns the result
 
         :param callable func: function with two arguments: one for the key, another for the value
@@ -434,7 +471,7 @@ class SuperDict(dict, Generic[K, V], Mapping[K, V]):
         """
         return SuperDict({k: func(k, v, *args, **kwargs) for k, v in self.items()})
 
-    def vapply(self, func: Callable, *args, **kwargs) -> "SuperDict":
+    def vapply(self, func: Callable[[V], R], *args, **kwargs) -> "SuperDict[K, R]":
         """
         Same as apply but only on values
 
@@ -443,7 +480,7 @@ class SuperDict(dict, Generic[K, V], Mapping[K, V]):
         """
         return SuperDict({k: func(v, *args, **kwargs) for k, v in self.items()})
 
-    def kapply(self, func: Callable, *args, **kwargs) -> "SuperDict":
+    def kapply(self, func: Callable[[K], R], *args, **kwargs) -> "SuperDict[K, R]":
         """
         Same as apply but only on keys
 
@@ -452,19 +489,27 @@ class SuperDict(dict, Generic[K, V], Mapping[K, V]):
         """
         return SuperDict({k: func(k, *args, **kwargs) for k in self})
 
-    def sapply(self, func: Callable, other: dict, *args, **kwargs) -> "SuperDict":
+    def sapply(
+        self,
+        func: Callable[[V], R],
+        other: Union[dict, int, float, str],
+        *args,
+        **kwargs,
+    ) -> "SuperDict[K, R]":
         """
         Applies function to both dictionaries.
         Using keys of the self.
         It's like applying a function over the left join.
 
         :param callable func: function to apply.
-        :param dict other: dictionary to apply function to
+        :param Union[dict, int, float,s tr] other: either an int, a float, a string or another dictionary to
+          perform the operation over
         :return: new :py:class:`SuperDict`
         """
-        return SuperDict(
-            {k: func(v, other[k], *args, **kwargs) for k, v in self.items()}
-        )
+        if isinstance(other, (int, float, str)):
+            return self.vapply(lambda v: func(v, other, *args, **kwargs))
+
+        return self.kvapply(lambda k, v: func(v, other[k], *args, **kwargs))
 
     def get_m(self, *args, default=None) -> Any:
         """
@@ -546,7 +591,7 @@ class SuperDict(dict, Generic[K, V], Mapping[K, V]):
                 "Pandas is not present in your system.\nTry: pip install pandas"
             )
 
-    def reverse(self):
+    def reverse(self) -> "SuperDict[V, K]":
         return SuperDict({v: k for k, v in self.items()})
 
     @classmethod
@@ -564,7 +609,7 @@ class SuperDict(dict, Generic[K, V], Mapping[K, V]):
             data[key] = cls.from_dict(value)
         return data
 
-    def copy_shallow(self) -> "SuperDict":
+    def copy_shallow(self) -> "SuperDict[K, V]":
         """
         Copies the immediate keys only.
 
@@ -572,13 +617,13 @@ class SuperDict(dict, Generic[K, V], Mapping[K, V]):
         """
         return SuperDict(self)
 
-    def copy_deep(self) -> "SuperDict":
+    def copy_deep(self) -> "SuperDict[K, V]":
         """
         Copies the complete object using python's pickle
         """
         return pickle.loads(pickle.dumps(self, -1))
 
-    def copy_deep2(self) -> "SuperDict":
+    def copy_deep2(self) -> "SuperDict[K, V]":
         """
         Copies the complete object using json (or ujson if available)
         """
