@@ -1,5 +1,5 @@
 import csv
-from typing import Callable, Iterable, Union, TypeVar, Generic, TYPE_CHECKING
+from typing import Callable, Iterable, Union, TypeVar, Generic, TYPE_CHECKING, List, Set
 import pickle
 from itertools import chain
 
@@ -9,10 +9,10 @@ if TYPE_CHECKING:
 from .tools import is_really_iterable
 
 T = TypeVar("T")
+R = TypeVar("R")
 
 
 class TupList(list, Generic[T]):
-
     """
     A list of tuples or dictionaries
     """
@@ -89,7 +89,7 @@ class TupList(list, Generic[T]):
             return TupList(x[0] for x in arr_filt)
         return TupList(tuple(x) for x in arr_filt)
 
-    def vfilter(self, function: Callable) -> "TupList":
+    def vfilter(self, function: Callable) -> "TupList[T]":
         """
         returns new list with only tuples for which `function` returns True
 
@@ -199,7 +199,7 @@ class TupList(list, Generic[T]):
         """
         return self.append(tuple(args))
 
-    def unique(self, **kwargs) -> "TupList":
+    def unique(self, **kwargs) -> "TupList[T]":
         """
         Applies :py:func:`numpy.unique`.
 
@@ -214,7 +214,7 @@ class TupList(list, Generic[T]):
         except ImportError:
             return self.unique2()
 
-    def unique2(self) -> "TupList":
+    def unique2(self) -> "TupList[T]":
         """
         Converts to set and then back to TupList.
 
@@ -222,7 +222,7 @@ class TupList(list, Generic[T]):
         """
         return TupList(set(self))
 
-    def intersect(self, input_list: Iterable) -> "TupList":
+    def intersect(self, input_list: Iterable) -> "TupList[T]":
         """
         Converts list and argument into sets and then intersects them.
 
@@ -231,7 +231,7 @@ class TupList(list, Generic[T]):
         """
         return TupList(set(self) & set(input_list))
 
-    def set_diff(self, input_list: Iterable) -> "TupList":
+    def set_diff(self, input_list: Iterable) -> "TupList[T]":
         """
         Converts list and argument into sets and then subtracts one from the other.
 
@@ -284,14 +284,14 @@ class TupList(list, Generic[T]):
         res_start_finish = [join_func(list_tup) for list_tup in all_periods]
         return TupList(res_start_finish)
 
-    def to_list(self) -> list:
+    def to_list(self) -> List[T]:
         """
 
         :return: list
         """
         return list(self)
 
-    def to_set(self) -> set:
+    def to_set(self) -> Set[T]:
         """
 
         :return: set
@@ -312,7 +312,7 @@ class TupList(list, Generic[T]):
         """
         return len(self)
 
-    def kvapply(self, func: Callable, *args, **kwargs) -> "TupList":
+    def kvapply(self, func: Callable[[int, T], R], *args, **kwargs) -> "TupList[R]":
         """
         maps function into each element of TupList with indexes
 
@@ -321,7 +321,7 @@ class TupList(list, Generic[T]):
         """
         return TupList(func(k, v, *args, **kwargs) for k, v in enumerate(self))
 
-    def kapply(self, func: Callable, *args, **kwargs) -> "TupList":
+    def kapply(self, func: Callable[[int], R], *args, **kwargs) -> "TupList[R]":
         """
         maps function into each key of TupList
 
@@ -330,7 +330,7 @@ class TupList(list, Generic[T]):
         """
         return TupList(func(k, *args, **kwargs) for k, _ in enumerate(self))
 
-    def vapply(self, func: Callable, *args, **kwargs) -> "TupList":
+    def vapply(self, func: Callable[[T], R], *args, **kwargs) -> "TupList[R]":
         """
         maps function into each element of TupList
 
@@ -349,7 +349,19 @@ class TupList(list, Generic[T]):
                 "Pandas is not present in your system. Try: pip install pandas"
             )
 
-    def sorted(self, **kwargs) -> "TupList":
+    @classmethod
+    def from_df(cls, data, **kwargs):
+        try:
+            import pandas as pd
+
+            return cls(data.to_dict(orient="records", **kwargs))
+
+        except ImportError:
+            raise ImportError(
+                "Pandas is not present in your system. Try: pip install pandas"
+            )
+
+    def sorted(self, **kwargs) -> "TupList[T]":
         """
         Applies sorted function to elements and returns a TupList
 
@@ -367,24 +379,40 @@ class TupList(list, Generic[T]):
         """
         return TupList(chain(*self))
 
-    def to_csv(self, path: str) -> "TupList":
+    def to_csv(self, path: str, header: list = None) -> "TupList":
         """
         Exports the list to a csv file
         :param path: filename
+        :param header: list of strings to use as header/column names for dict
         :return: the same :py:class:`TupList`
         """
+        if len(self) == 0:
+            return self
+        first = self[0]
+        if header is not None and len(first) != len(header):
+            raise ValueError("Header length does not match data length")
+        # Handle case of list of dict
+        if isinstance(first, dict):
+            if header is None:
+                header = first.keys()
+            thing_to_write = self.take(list(header))
+        else:
+            thing_to_write = self
         with open(path, "w", newline="\n", encoding="utf-8") as out:
             csv_out = csv.writer(out)
-            csv_out.writerows(self)
+            if header is not None:
+                csv_out.writerow(header)
+            csv_out.writerows(thing_to_write)
         return self
 
     @classmethod
     def from_csv(cls, path: str, func: Callable = None, **kwargs) -> "TupList":
         """
         Generates a new TupList by reading a csv file
-        :param path: filename
+
+        :param str path: filename
         :param callable func: function to apply to each row
-        :**kwargs: arguments to csv.reader
+        :param kwargs: arguments to csv.reader
         :return: new :py:class:`TupList`
         """
         if func is None:
@@ -400,18 +428,21 @@ class TupList(list, Generic[T]):
         :param callable func: function to apply to create col
         """
 
-        def apply_to_tub(my_tuple):
+        def apply_to_tup(my_tuple):
+            # we apply the function before any potential modification
+            result = func(my_tuple)
             # if it's un-mutable (tuple), we need to make it a list
             tuple_flag = 0
             if isinstance(my_tuple, tuple):
                 tuple_flag = 1
                 my_tuple = list(my_tuple)
+            # if None, we assume we want it at the end
             if tuple_flag and pos is None:
-                my_tuple.append(func(my_tuple))
+                my_tuple.append(result)
             else:
-                my_tuple[pos] = func(my_tuple)
+                my_tuple[pos] = result
             if tuple_flag:
                 my_tuple = tuple(my_tuple)
             return my_tuple
 
-        return self.vapply(apply_to_tub)
+        return self.vapply(apply_to_tup)
